<?php

namespace Litecms\Memo\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Memo\Forms\Memo as MemoForm;
use Litecms\Memo\Http\Requests\MemoResourceRequest;
use Litecms\Memo\Http\Resources\MemoResource;
use Litecms\Memo\Http\Resources\MemosCollection;
use Litecms\Memo\Models\Memo;
use Litecms\Memo\Actions\MemoAction;
use Litecms\Memo\Actions\MemoActions;


/**
 * Resource controller class for memo.
 */
class MemoResourceController extends BaseController
{

    /**
     * Initialize memo resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->middleware(function ($request, $next) {
            $this->form = MemoForm::grouped(false)
                            ->setAttributes()
                            ->toArray();
            $this->modules = $this->modules(config('litecms.memo.modules'), 'memo', guard_url('memo'));
            return $next($request);
        });
    }

    /**
     * Display a list of memo.
     *
     * @return Response
     */
    public function index(MemoResourceRequest $request)
    {
        $request = $request->all();
        $page = MemoActions::run('paginate', $request);

        $data = new MemosCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('memo::memo.names'))
            ->view('memo::memo.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display memo.
     *
     * @param Request $request
     * @param Model   $memo
     *
     * @return Response
     */
    public function show(MemoResourceRequest $request, Memo $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MemoResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('memo::memo.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('memo::memo.show')
            ->output();
    }

    /**
     * Show the form for creating a new memo.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(MemoResourceRequest $request, Memo $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MemoResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('memo::memo.name'))
            ->view('memo::memo.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new memo.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(MemoResourceRequest $request, Memo $model)
    {
        try {
            $request = $request->all();
            $model = MemoAction::run('store', $model, $request);
            $data = new MemoResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('memo::memo.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('memo/memo/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/memo/memo'))
                ->redirect();
        }

    }

    /**
     * Show memo for editing.
     *
     * @param Request $request
     * @param Model   $memo
     *
     * @return Response
     */
    public function edit(MemoResourceRequest $request, Memo $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new MemoResource($model);
        // return view('memo::memo.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('memo::memo.name'))
            ->view('memo::memo.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the memo.
     *
     * @param Request $request
     * @param Model   $memo
     *
     * @return Response
     */
    public function update(MemoResourceRequest $request, Memo $model)
    {
        try {
            $request = $request->all();
            $model = MemoAction::run('update', $model, $request);
            $data = new MemoResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('memo::memo.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('memo/memo/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('memo/memo/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the memo.
     *
     * @param Model   $memo
     *
     * @return Response
     */
    public function destroy(MemoResourceRequest $request, Memo $model)
    {
        try {

            $request = $request->all();
            $model = MemoAction::run('destroy', $model, $request);
            $data = new MemoResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('memo::memo.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('memo/memo/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('memo/memo/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
